Type.registerNamespace("asu.utils.xml");

/**
 * 接口，根据DataSource产生XMLDom对象
 * @constructor
 * @param {Object} datasource
 */
asu.utils.xml.IDOMProvider = function(datasource){
	this.createDom = Function.abstractMethod;
}

Type.registerInterface("asu.utils.xml.IDOMProvider");

/**
 * IDOMProvider的默认实现类，根据数据岛或者XML的Url来返回XMLDom对象
 * 也可根据html控件来自动生成xmlDom对象
 * @param {Object} datasource
 */
asu.utils.xml.GeneralDOMProvider = function(datasource){

	var SRCTYPE_XML     = 1;           // 数据岛
	var SRCTYPE_OBJECT  = 2;           // html控件
	var SRCTYPE_URL     = 3;           // 通过URL来获取的数据源
	var SRCTYPE_MISSING = 0;           // 不支持的数据源

	var _datasource     = datasource;  // 数据源
	var _dom            = null;        // 根据数据源产生的dom
	var _type           = null;        // 数据源类型

	/**
	 * 创建dom文档
	 */
	this.createDom = function(){

		if (_dom != null)
			return _dom;

		if (_datasource == null) {
			throw new Error("数据源不能为空");
		}

		_type = getDataSourceType();

		if (_type == SRCTYPE_MISSING) {
			throw new Error("不正确的数据源格式");
		} else if (_type == SRCTYPE_XML) {
			_dom = _datasource.XMLDocument;
		} else if (_type == SRCTYPE_OBJECT) {
			var controlHelper = new asu.utils.xml.ControlHelper(_datasource);
			_dom = controlHelper.getDom(false);
		}

		return _dom;
	}

	/**
	 * 获取数据源类型
	 */
	var getDataSourceType = function() {

		if (_datasource.XMLDocument != undefined) {

			// 数据岛数据源
			return SRCTYPE_XML;

		} else if (_datasource.tagName != undefined) {

			// html控件数据源
			if (asu.utils.xml.ControlHelper.validateContainer(_datasource)) {
				return SRCTYPE_OBJECT;
			} else {
				return SRCTYPE_MISSING;
			}

		} else {

			// url数据源
			try {
				_dom = new asu.utils.xml.BRequest().sendRequest(_datasource, '', null, false).responseXML.getInnerDocument();
				return SRCTYPE_URL;
			} catch (e) {
				alert(e.description);
				return SRCTYPE_MISSING;
			}

		}
	}
}

asu.utils.xml.GeneralDOMProvider.registerClass('asu.utils.xml.GeneralDOMProvider',asu.utils.xml.IDOMProvider);

/**
 * @constructor
 * @param {String} obj html控件
 * @param {bool} container 是否为容器
 */
asu.utils.xml.HtmlControl = function(obj, container){
	//var _id 	     = id;
	var _object      = obj;
	var _parent      = null;
	var _cnodes      = null;
	//值数据所对应的XML节点
	var _xPath     = (undefined == _object.xpath) ? "" : _object.xpath;
	//显示内容所对应的XML节点,如果_displayFld为空，则显示和值所对应节点相同
	var _displayFld  = (undefined == _object.displayFld) ? _xPath : _object.displayFld;
	//数据绑定时，是否需要进行去空字符串的处理
	var _trimSpace   = (undefined == _object.trimSpace) ? true : (_object.trimSpace.toUpperCase() == "TRUE" ? true : false);

	// 是否是容器
	this.isContainer = container;
	//alert(this.isContainer);
	/**
	 * @return {HTML Element} 返回HTML页面中指定id的Element
	 */
	this.toObject = function(){
		return _object;
	}
	//alert(this.toObject().value);
	/**
	 * @return {boolean} 获取HtmlControl的值时，是否去处空字符,默认为True
	 */
	this.getTrimSpace = function(){
		return _trimSpace;
	}

	/**
	 * 设定获取HtmlControl的值时是否需要去处空字符。默认情况下不用
	 * 调用该函数，而是在构建HtmlControl时，自动从所对应的Html
	 * Element中读取trimSpace属性获取该相应值
	 * @param {Object} value
	 */
	this.setTrimSpace = function(value){
		_trimSpace = value;
	}

	/**
	 * @return {String} HtmlControl的value所对应的XML Dom节点的Xpath
	 */
	this.getNode = function(){
		// 父节点的xpath
		var parentPath = (_parent != null ? _parent.getNode() : "");

		if (_xPath != "") {
			return parentPath + "/" + _xPath;
		}

		return parentPath;
	}

	/**
	 * @return {String} HtmlControl的显示值所对应的XML Dom节点的Xpath
	 */
	this.getDisplayNode = function(){
		// 父节点的xpath
		var parentPath = (_parent != null ? _parent.getNode() : "");

		return parentPath + "/" + _displayFld;
	}

	this.getParent = function(){
		return _parent;
	}

	/**
	 * 添加父节点
	 */
	this.setParent = function(p){
		_parent = p;
	}

	/**
	 * 添加子节点
	 */
	this.addChildNode = function(control) {
		if (control == null) {
			alert("添加子节点不能为空！");
			return;
		}

		if (_cnodes == null) {
			_cnodes = new Array();
		}

		_cnodes.insert(_cnodes.length, control);
	}

	this.getCNodes = function() {
		return _cnodes;
	}

	this.getXpath = function() {
		return _xPath;
	}

	this.getValue = Function.abstractMethod;
	this.setValue = Function.abstractMethod;

	//alert("end");
}

Type.registerClass("asu.utils.xml.HtmlControl");

/**
 * 带value属性的控件
 */
asu.utils.xml.CommonHtmlControl = function(obj, container){
	// 初始化子控件
	asu.utils.xml.CommonHtmlControl.initializeBase(this, [obj, container]);

	this.getValue = function(){
		return this.toObject().value;
	}

	this.setValue = function(dom){
		//var value;
		// TODO: 这里设置的值应该是显示值,暂时先设为_xPath对应的节点值,需要扩展时修改这里
		if (dom != null && dom.xml != undefined && this.getXpath() != "") {
			var v = dom.selectSingleNode(this.getNode());

			if (v != null && v.text.trim() != "") {
				this.toObject().value = v.text;
			}
		} else {
			//alert("错误的dom参数");
			return;
		}
	}
}

asu.utils.xml.CommonHtmlControl.registerClass("CommonHtmlControl",asu.utils.xml.HtmlControl);

/**
 * Checkbox控件
 */
asu.utils.xml.CheckboxControl = function(obj, container){
	// 初始化子控件
	asu.utils.xml.CheckboxControl.initializeBase(this, [obj, container]);

	/**
	 * @return {String} HtmlControl的value所对应的XML Dom节点的Xpath
	 */
	this.getNode = function(){
		return this.getParent().getNode();
	}

	/**
	 * @return {String} HtmlControl的显示值所对应的XML Dom节点的Xpath
	 */
	this.getDisplayNode = function(){
		return this.getParent().getDisplayNode();
	}

	// 返回是否被选中
	this.getValue = function(){
		return this.toObject().checked;
	}

	this.setValue = function(value){
		this.toObject().checked = value;
	}
}

asu.utils.xml.CheckboxControl.registerClass("CheckboxControl",asu.utils.xml.HtmlControl);

/**
 * radio控件
 */
asu.utils.xml.RadioControl = function(obj, container){
	// 初始化子控件
	asu.utils.xml.RadioControl.initializeBase(this, [obj, container]);

	/**
	 * @return {String} HtmlControl的value所对应的XML Dom节点的Xpath
	 */
	this.getNode = function(){
		return this.getParent().getNode();
	}

	/**
	 * @return {String} HtmlControl的显示值所对应的XML Dom节点的Xpath
	 */
	this.getDisplayNode = function(){
		return this.getParent().getDisplayNode();
	}

	// 返回是否被选中
	this.getValue = function(){
		return this.toObject().checked;
	}

	this.setValue = function(value){
		this.toObject().checked = value;
	}
}

asu.utils.xml.RadioControl.registerClass("RadioControl",asu.utils.xml.HtmlControl);


/**
 * 容器代表着一个HTML的标签，该标签下包含了XSQL的UIControl
 * 可以为table,div,form等容器
 * @constructor
 * @author knight
 */
asu.utils.xml.HtmlContainer = function(obj, container){
	// 初始化子控件
	asu.utils.xml.HtmlContainer.initializeBase(this, [obj, container]);

	this.getValue = function(){
		return this.toObject().value == undefined ? "" : this.toObject().value;
	}

	/* 给子节点赋值 */
	this.setValue = function(dom){
		var _cnodes = this.getCNodes();

		if (_cnodes == null)
			return;

		var node;

		for (var i = 0; i < _cnodes.length; i++) {
			node = _cnodes[i];
			node.setValue(dom);
		}
	}
}

asu.utils.xml.HtmlContainer.registerClass('asu.utils.xml.HtmlContainer',asu.utils.xml.HtmlControl);


/**
 * 一个checkbox容器,包含同组checkbox控件
 * @constructor
 * @author knight
 */
asu.utils.xml.CheckboxContainer = function(obj, container, n){
	this.split = ",";
	this._name = n;
	// 初始化子控件
	asu.utils.xml.CheckboxContainer.initializeBase(this, [obj, container]);

	this.getValue = function(){
		var _cnodes = this.getCNodes();

		if (_cnodes == null)
			return;

		var node;
		var value = "";
		for (var i = 0; i < _cnodes.length; i++) {
			node = _cnodes[i];

			if (node.toObject().checked)
				value += node.toObject().value + this.split;
		}

		// 去掉最后的split
		return value == "" ? "" : value.substr(0, value.length - 1);
	}

	/* 给子节点赋值 */
	this.setValue = function(dom){

		var value;

		try {
			if (dom != null && dom.xml != undefined) {
				value = dom.selectSingleNode(this.getNode()).text;
			} else {
				alert("错误的dom参数");
				return;
			}

			var _cnodes = this.getCNodes();

			if (_cnodes == null)
				return;

			// value的模式为"value1,value2,value3"
			var strs = value.split(this.split);
			var node;
			for (var i = 0; i < _cnodes.length; i++) {
				node = _cnodes[i];
				for(var j = 0; j < strs.length; j++) {
					if (node.toObject().value == strs[j])
						node.setValue(true);
				}
			}

		} catch (e) {
			alert(e.description);
			return;
		}
	}
}

asu.utils.xml.CheckboxContainer.registerClass('asu.utils.xml.CheckboxContainer',asu.utils.xml.HtmlContainer);

/**
 * 一个radio容器,包含同组radio控件
 * @constructor
 * @param obj = null, container指明是否为container, n为单选框组对应的name
 * @author knight
 */
asu.utils.xml.RadioContainer = function(obj, container, n){
	this._name = n;
	// 初始化子控件
	asu.utils.xml.RadioContainer.initializeBase(this, [obj, container]);

	this.getValue = function(){
		var _cnodes = this.getCNodes();

		if (_cnodes == null)
			return;

		var node;
		var value = "";
		for (var i = 0; i < _cnodes.length; i++) {
			node = _cnodes[i];

			if (node.toObject().checked) {
				return node.toObject().value;
			}

		}

		// 没有选中时返回空串
		return value;
	}

	/* 给子节点赋值 */
	this.setValue = function(dom){

		var value;

		try {
			if (dom != null && dom.xml != undefined) {
				value = dom.selectSingleNode(this.getNode()).text;
			} else {
				alert("错误的dom参数");
				return;
			}

			var _cnodes = this.getCNodes();

			if (_cnodes == null)
				return;

			var node;
			for (var i = 0; i < _cnodes.length; i++) {
				node = _cnodes[i];
				if (node.toObject().value == value) {
					node.setValue(true);
					return;
				}
			}

		} catch (e) {
			alert(e.description);
			return;
		}
	}
}

asu.utils.xml.RadioContainer.registerClass('asu.utils.xml.RadioContainer',asu.utils.xml.HtmlContainer);

/**
 * 可以根据根容器生成控件树和xml文档等
 * getDom--返回xml
 * getControlTree--返回以_root为根容器的控件树
 */
asu.utils.xml.ControlHelper = function(obj) {

	var _root;          // 根容器
	//var _xml;
	var _tree;
	var _dom;
	var _flag = false;  // 是否需要生成数据

	var init = function(o) {
		if (!asu.utils.xml.ControlHelper.validateContainer(o))
			throw new Error(null, "此控件不能作为根容器");

		_root = o;
		//_xml = null;
		_tree = null;
		_dom = null;
	}

	// 初始化
	init(obj);

	// 获取根容器
	this.getRoot = function() {
		return _root;
	}

	// 创建dom结构,只创建一次
	this.getDom = function(flag) {
		_flag = (flag != null ? flag : false);

		try {

			this.getControlTree();
			_dom = new asu.utils.xml.BDocument("<root/>");
			travelCNode(_dom.getRoot(), _tree);
			_dom = new asu.utils.xml.BDocument(_dom.getRoot().childNodes[0].xml);
			return _dom.getInnerDocument();

		} catch (e) {
			alert(e.description);
			return null;
		}
	}

	// 遍历控件树生成xml
	var travelCNode = function(parentNode, control) {
		var node = createNode(parentNode, control);

		if (!control.isContainer && _flag) {
			var text = _dom.getInnerDocument().createTextNode(control.getValue());
			node.appendChild(text);
		} else if (asu.utils.xml.RadioContainer.isInstanceOfType(control)
				|| asu.utils.xml.CheckboxContainer.isInstanceOfType(control)) {

			if (_flag) {
				var text = _dom.getInnerDocument().createTextNode(control.getValue());
				node.appendChild(text);
			}

			return;
		}

		var cnodes = control.getCNodes();

		if (cnodes != null && cnodes.length > 0) {
			for (var i = 0; i < cnodes.length; i++) {
				travelCNode(node, cnodes[i]);
			}
		}
	}

	// 获取父节点的xpath
	var getParentXpath = function(par) {
		if (par.getParent() != null) {
			return par.getParent().getNode() + "/";
		} else {
			return "/root/";
		}
	}

	var createNode = function(parentNode, control) {
		var strs = control.getXpath().split('/');
		var path = getParentXpath(control);
		var node = parentNode;
		var preNode = parentNode;

		for (var i = 0; i < strs.length; i++) {
			if (strs[i] != '') {
				path += strs[i];

				node = _dom.selectSingleNode(path);

				if (node == null) {
					node = _dom.appendNodeAt(preNode, strs[i], "");
					preNode = node;
				}
				path += "/";
			}
		}

		return node;
	}

	// 生成控件树,只生成一次
	this.getControlTree = function() {
		try {
			if (_tree != null)
				return _tree;
			else {
				// 创建根容器
				_tree = new asu.utils.xml.HtmlContainer(_root, true);
				travelTree(_tree);

				return _tree;
			}
		} catch (e) {
			alert(e.description);
			return null;
		}
	}

	/**
	 * 遍历子控件
	 * @param p 父控件
	 */
	var travelTree = function(p) {
		var obj = p.toObject();
		var childNodes;
		var nextNode;

		if (obj.children != undefined && obj.children.length > 0) {
			childNodes = obj.children;

			for (var i = 0; i < childNodes.length; i++) {
				nextNode = null;
				var node = childNodes[i];
				var type = asu.utils.xml.ControlHelper.getControlType(node);

				if (type == asu.utils.xml.ControlHelper.T_MISSING) { // 无效的类型
					continue;
				} else if (type == asu.utils.xml.ControlHelper.T_CONTAINER) { // 容器
					nextNode = new asu.utils.xml.HtmlContainer(node, true);
					nextNode.setParent(p);
					p.addChildNode(nextNode);
				} else if (type == asu.utils.xml.ControlHelper.T_COMMONCONTROL) { // 普通控件
					nextNode = new asu.utils.xml.CommonHtmlControl(node, false);
					nextNode.setParent(p);
					p.addChildNode(nextNode);
					nextNode = null;
				} else if (type == asu.utils.xml.ControlHelper.T_CHECKBOX) { // 复选框
					var _checkboxContainer = getCheckboxContainer(p, node, node.name);
					nextNode = new asu.utils.xml.CheckboxControl(node, false);
					_checkboxContainer.addChildNode(nextNode);
					nextNode = null;
				} else if (type == asu.utils.xml.ControlHelper.T_RADIO) { // 单选框
					var _radioContainer = getRadioContainer(p, node, node.name);
					nextNode = new asu.utils.xml.RadioControl(node, false);
					_radioContainer.addChildNode(nextNode);
					nextNode = null;
				}

				if (nextNode != null)
					travelTree(nextNode);
			}
		}
	}

	/**
	 * 获取控件p指定名字为n的CheckboxContainer,如果已经创建则返回原来的CheckboxContainer
	 * 否则创建一个新的CheckboxContainer
	 */
	var getCheckboxContainer = function(p, obj, n) {
		var _container = null;

		if (p.getCNodes() == null) {

			_container = new asu.utils.xml.CheckboxContainer(obj, true, n);
			_container.setParent(p);
			p.addChildNode(_container);

			return _container;
		}

		var child = null;
		var cnodes = p.getCNodes();
		for (var i = 0; i < cnodes.length; i++) {
			child = cnodes[i];

			if (child._name != undefined && child._name == n)
				return child;
		}

		_container = new asu.utils.xml.CheckboxContainer(obj, true, n);
		_container.setParent(p);
		p.addChildNode(_container);
		return _container;
	}

	/**
	 * 获取控件p指定名字为n的RadioContainer,如果已经创建则返回原来的RadioContainer
	 * 否则创建一个新的RadioContainer
	 */
	var getRadioContainer = function(p, obj, n) {
		var _container = null;

		if (p.getCNodes() == null) {
			_container = new asu.utils.xml.RadioContainer(obj, true, n);
			_container.setParent(p);
			p.addChildNode(_container);
			return _container;
		}

		var child = null;
		var cnodes = p.getCNodes();
		for (var i = 0; i < cnodes.length; i++) {
			child = cnodes[i];

			if (child._name != undefined && child._name == n)
				return child;
		}

		_container = new asu.utils.xml.RadioContainer(obj, true, n);
		_container.setParent(p);
		p.addChildNode(_container);
		return _container;
	}

}
asu.utils.xml.ControlHelper.registerClass('asu.utils.xml.ControlHelper');

// 判断是否为合法的container
asu.utils.xml.ControlHelper.validateContainer = function(o) {
	var tagName = (o == null ? "" : o.tagName.toUpperCase());

	// 如果o不能作为根容器,初始化失败
	if (o == null || ("TABLE" != tagName && "FORM" != tagName && "DIV" != tagName && "TR" != tagName && "TD" != tagName && "TBODY" != tagName))
		return false;

	return true;
}

// 判断是否为合法的控件
asu.utils.xml.ControlHelper.validateControl = function(o) {
	var tagName = (o == null ? "" : o.tagName.toUpperCase());
	var _names = new Array("TABLE", "FORM", "DIV", "SPAN", "INPUT",
							"TEXTAREA", "SELECT", "TR", "TD", "TBODY");

	if (o == null)
		return false;

	for (var i = 0; i < _names.length; i++) {
		if ( tagName == _names[i])
			return true;
	}


	return false;
}

// 控件的类型
asu.utils.xml.ControlHelper.T_MISSING       = 0;  // 无效
asu.utils.xml.ControlHelper.T_CONTAINER     = 1;  // 容器
asu.utils.xml.ControlHelper.T_COMMONCONTROL = 2;  // 普通具有value值的控件,不包括checkbox和radio
asu.utils.xml.ControlHelper.T_CHECKBOX      = 3;  // 复选框
asu.utils.xml.ControlHelper.T_RADIO         = 4;  // 单选框


// 判断控件的类型
asu.utils.xml.ControlHelper.getControlType = function(o) {

	if (!asu.utils.xml.ControlHelper.validateControl(o)) {
		return asu.utils.xml.ControlHelper.T_MISSING;
	} else if (asu.utils.xml.ControlHelper.validateContainer(o)) {
		return asu.utils.xml.ControlHelper.T_CONTAINER;
	} else {
		if (o.tagName.toUpperCase() == "INPUT" && o.type.toUpperCase() == "CHECKBOX")
			return asu.utils.xml.ControlHelper.T_CHECKBOX;
		else if (o.tagName.toUpperCase() == "INPUT" && o.type.toUpperCase() == "RADIO")
			return asu.utils.xml.ControlHelper.T_RADIO;
		else
			return asu.utils.xml.ControlHelper.T_COMMONCONTROL;
	}

	return asu.utils.xml.ControlHelper.T_MISSING;
}

/**
 * 该类具体实现XML数据源和Html页面的控件绑定
 * @constructor
 * @param {IDOMProvider} DataBinder数据源的提供者
 * @param {root} 根容器控件的id
 * @example: 我们可以编写以下的Html页面：
 *   <input id="test" node="user/name"/>
 *   <select id="test" node="user/age">
 *   	<option value = "20">二十</option>
 *      <option value = "21">二十一</option>
 *      <option value = "22">二十二</option>
 *   </select>
 *   <xml id = "user">
 *   	<user>
 *      	<name>wuyang</name>
 *          <age>22</age>
 *      </user>
 *   </xml>
 */
asu.utils.xml.DataBinder = function(DOMProvider){
	var _dom		 = null;
	var _DOMProvider = null;
	var _root        = null;
	var _tree        = null;
	var _ctrlHelper  = null;

	if (arguments.length >= 1){
		//if (!asu.utils.xml.IDOMProvider.isImplementedBy(DOMProvider))
		//	throw new Error("不正确的DOM提供者");

		_DOMProvider = DOMProvider;
		//_root        = root;

		try {
			_dom        = _DOMProvider.createDom();
		} catch (e) {
			alert(e.description);
		}

	}
	else
		throw new Error("请提供dom提供者");

	this.getDOMProvider = function(){
		return _DOMProvider
	}

	this.getDOM = function() {
		return _dom;
	}

	/**
	 *
	 * 根容器的初始化,在做bind和getDataSource之前
	 * 都要先初始化根容器
	 * @param root 绑定xml数据源的根容器
	 */
	this.initRoot = function(root) {
		_ctrlHelper = new asu.utils.xml.ControlHelper(root);
		_tree       = _ctrlHelper.getControlTree();
	}

	/**
	 * 将XML数据源填充到HTML页面的Element中
	 */
	this.bind = function(){
		if (_tree == null) {
			throw new Error("根容器还没有初始化!");
		}

		_tree.setValue(_dom);
	}

	/**
	 * 从Html页面的Element中获取数据，并更新XML数据源
	 */
	this.getDataSource = function(){
		if (_tree == null) {
			throw new Error("根容器还没有初始化!");
		}

		travelCNode(_tree);

		return _dom;
	}

	// 遍历控件树生成xml
	var travelCNode = function(control) {
		if (control == null)
			return;

		var node;         // 控件对应的节点
		var nodeText;     // 节点值
		var cn;           // 子节点

		if (!control.isContainer
				|| asu.utils.xml.RadioContainer.isInstanceOfType(control)
				|| asu.utils.xml.CheckboxContainer.isInstanceOfType(control))
		{
			if (control.getXpath() != "" && _dom.selectSingleNode(control.getNode()) != null) {

				node         = _dom.selectSingleNode(control.getNode());
				nodeText     = _dom.createTextNode(control.getTrimSpace() ? control.getValue().trim() : control.getValue());

				// 清空节点的文本
				for (var j = 0; j < node.childNodes.length; j++) {
					cn = node.childNodes[j];
					if (cn.nodeType == 3) {// 文本节点
						node.removeChild(cn);
						break;
					}
				}

				node.appendChild(nodeText);
			}
			return;
		}

		var cnodes = control.getCNodes();

		if (cnodes != null && cnodes.length > 0) {
			for (var i = 0; i < cnodes.length; i++) {
				travelCNode(cnodes[i]);
			}
		}
	}
}

asu.utils.xml.DataBinder.registerClass('asu.utils.xml.DataBinder');